from charmhelpers.core import (
    hookenv,
    templating
)

import os
import subprocess
import filecmp
import netifaces
import ipaddress


def get_iface_ip_in_subnet(subnet):
    matched_ip = ""

    if subnet != "":
        net = ipaddress.ip_network(subnet)

        all_ifs = netifaces.interfaces()
        for if_item in all_ifs:
            netifaces.ifaddresses(if_item)
            if netifaces.AF_INET in netifaces.ifaddresses(if_item).keys():
                ip = netifaces.ifaddresses(if_item)[netifaces.AF_INET][0]['addr']
                ip_addr = ipaddress.ip_address(ip)
                if ip_addr in net.hosts():
                    matched_ip = ip

    return matched_ip


FILES_TO_REMOVE = [
    "/etc/nginx/sites-enabled/netdata",
    "/etc/nginx/sites-available/netdata",
]


def remove_config():
    hookenv.log("*** Delete files: ", "DEBUG")
    for file_to_remove in FILES_TO_REMOVE:
        hookenv.log(
            "*** File to delete: {}".format(file_to_remove),
            "DEBUG"
        )
        if os.path.exists(file_to_remove):
            os.unlink(file_to_remove)
            hookenv.log(
                "*** File deleted!: {}".format(file_to_remove),
                "DEBUG"
            )

    subprocess.run("systemctl restart nginx", shell=True, check=True)


def configure_nginx_netdata_vhost():
    config = hookenv.config()

    if config['netdata_nginx_allowed_subnet'] != "all":
        try:
            ipaddress.ip_network(config['netdata_nginx_allowed_subnet'])
        except ValueError:
            hookenv.log(
                "*** Not valid subnet given for "
                "'netdata_nginx_allowed_subnet'!: {}"
                .format(config['netdata_nginx_allowed_subnet']),
                "DEBUG"
            )
            raise

    if config['netdata_nginx_listen_subnet'] != "":
        try:
            ipaddress.ip_network(config['netdata_nginx_listen_subnet'])
        except ValueError:
            hookenv.log(
                "*** Not valid subnet given for "
                "'netdata_nginx_listen_subnet'!: {}"
                .format(config['netdata_nginx_listen_subnet']),
                "DEBUG"
            )
            raise

    nginx_listen_ip = get_iface_ip_in_subnet(config['netdata_nginx_listen_subnet'])

    templating.render(
        source='netdata_nginx_vhost',
        target='/etc/nginx/sites-available/netdata',
        context={
            "netdata_nginx_port":
                str(config['netdata_nginx_port']),
            "netdata_nginx_allowed_subnet":
                config['netdata_nginx_allowed_subnet'],
            "netdata_nginx_listen_ip": nginx_listen_ip
        },
        owner="root",
        group="root",
        perms=0o644
    )

    netdata_site_available_path = "/etc/nginx/sites-available/netdata"
    netdata_site_enabled_path = "/etc/nginx/sites-enabled/netdata"
    default_site_enabled_path = "/etc/nginx/sites-enabled/default"

    if os.path.exists(default_site_enabled_path):
        os.unlink(default_site_enabled_path)
        hookenv.log(
            "*** File deleted!: {}".format(default_site_enabled_path),
            "DEBUG"
        )

    if os.path.exists(netdata_site_enabled_path):
        if not filecmp.cmp(netdata_site_available_path,
                           netdata_site_enabled_path):
            os.unlink(netdata_site_enabled_path)
            hookenv.log(
                "*** File deleted!: {}".format(netdata_site_enabled_path),
                "DEBUG"
            )
            subprocess.run("ln -s /etc/nginx/sites-available/netdata "
                           "/etc/nginx/sites-enabled/netdata",
                           shell=True,
                           check=True)
    else:
        subprocess.run("ln -s /etc/nginx/sites-available/netdata "
                       "/etc/nginx/sites-enabled/netdata",
                       shell=True,
                       check=True)

    subprocess.run("systemctl restart nginx", shell=True, check=True)


def fix_rootfs_netdata_systemd_conf():

    def _replace_line(file_name, existing_line, new_line):
        new_file_lines = []
        with open(file_name, 'r') as f:
            file_lines = f.readlines()
            for line in file_lines:
                if line == existing_line + "\n":
                    new_file_lines.append(new_line + "\n")
                else:
                    new_file_lines.append(line)
        with open(file_name, 'w') as f:
            f.writelines(new_file_lines)

    _replace_line(
        '/etc/systemd/system/multi-user.target.wants/netdata.service',
        'ReadOnlyDirectories=/', '# ReadOnlyDirectories=/')

    subprocess.run("systemctl daemon-reload", shell=True, check=True)
    subprocess.run("systemctl restart netdata.service", shell=True, check=True)
